@@ -16,7 +16,17 @@ import sys
 import typing
 import warnings
 
-import tomllib
+try:
+    import tomllib
+except ImportError:
+    # Python < 3.11, no tomllib available
+    tomllib = None  # type: ignore
+
+APP_NAME = "sphinx-linter"
+
+# ============================================================================
+# Default ignored directories for file discovery
+# ============================================================================
 
 # Ignore, at least, these directory basenames (default directories to ignore).
 default_ignore_dirs = (
@@ -69,8 +79,8 @@ blank_lines_regex = re.compile("(?:^[ \t]*\r?\n){2,}(?=[^\r\n])", flags=re.MULTI
 # Docstring starting or ending with only quotes lines
 quotes_starts_regex = re.compile(r'^"+\s*$')
 quotes_ends_regex = re.compile(r'^\s*"+$')
-# Leading whitespaces matcher (ignoring break-line)
-leading_ws_regex = re.compile('^[ \t]+')
+# Leading whitespaces matcher: line starts with spaces/tabs followed by non-space
+leading_ws_regex = re.compile('^[ \t]+\\S+')
 
 # ----------------------------------
 # Bad whitespace in section left-side:
@@ -182,10 +192,11 @@ class ParsedDocs(typing.NamedTuple):
 
 
 class Violations:
-    # Rules definition:
-    #   Enabled, Code, Message
-
+    # ----------------------------------------------------------------------------
+    # Rules definition: Enabled | Code | Message
+    # ----------------------------------------------------------------------------
     # DOC0xx: Docstring section issues
+    # ----------------------------------------------------------------------------
     DOC001 = (True, "DOC001", "Invalid docstring section ({!r})")
     DOC002 = (True, "DOC002", "Malformed section ({!r})")
     DOC003 = (True, "DOC003", "Missing blank line after docstring")
@@ -203,8 +214,9 @@ class Violations:
     DOC010 = (True, "DOC010", "Section definition contains invalid whitespace ({!r})")
     DOC011 = (True, "DOC011", "Trailing non-empty lines after last section")
     DOC012 = (True, "DOC012", "Leading whitespaces in first non-blank line ({!r})")
-
+    # ----------------------------------------------------------------------------
     # DOC1xx: Parameter issues
+    # ----------------------------------------------------------------------------
     DOC101 = (True, "DOC101", "Parameter documented but not in signature ({!r})")
     DOC102 = (True, "DOC102", "Invalid parameter type syntax ({!r})")
     DOC103 = (True, "DOC103", "Parameter type already in signature ({!r})")
@@ -212,19 +224,22 @@ class Violations:
     DOC105 = (True, "DOC105", "Duplicated parameter ({!r})")
     DOC106 = (True, "DOC106", "Parameter order mismatch with signature")
     DOC107 = (False, "DOC107", "Missing parameter in docstring ({!r})")  # Disabled by default
-
+    # ----------------------------------------------------------------------------
     # DOC2xx: Return issues
+    # ----------------------------------------------------------------------------
     DOC201 = (True, "DOC201", "Return documented but function has no return statement")
     DOC202 = (True, "DOC202", "Invalid return type syntax ({!r})")
     DOC203 = (True, "DOC203", "Return type already in signature ({!r})")
     DOC204 = (True, "DOC204", "Return type mismatch with annotation ({!r} != {!r})")
     DOC205 = (True, "DOC205", "Duplicated return section ({!r})")
-
+    # ----------------------------------------------------------------------------
     # DOC3xx: Raises issues
+    # ----------------------------------------------------------------------------
     DOC302 = (True, "DOC302", "Invalid exception type syntax ({!r})")
     DOC305 = (True, "DOC305", "Duplicated exception type ({!r})")
-
+    # ----------------------------------------------------------------------------
     # DOC4xx: Variable issues
+    # ----------------------------------------------------------------------------
     DOC402 = (True, "DOC402", "Invalid variable type syntax ({!r})")
     DOC403 = (True, "DOC403", "Variable name contains invalid whitespace ({!r})")
     DOC405 = (True, "DOC405", "Duplicated variable ({!r})")
@@ -292,7 +307,7 @@ class Violations:
         if not first.strip() and parsed_docs.docs:
             lines = parsed_docs.docs.splitlines()
             first = lines[0] if lines else ""
-        if first and not first.isspace() and leading_ws_regex.search(first):
+        if first and leading_ws_regex.search(first):
             yield cls.DOC012, (first,)
 
         for section_def in parsed_docs.bad_whitespaces_def:
@@ -370,10 +385,10 @@ class Violations:
         # Check parameter order ignoring missing/extra parameters
         orderby = operator.attrgetter("order")
         # Ignore parameters not documented
-        defined_documented = [n.name for n in sorted(parsed_params.values(), key=orderby) if n.name in documented]
+        defined_documented = (n.name for n in sorted(parsed_params.values(), key=orderby) if n.name in documented)
         # Ignore documented parameters not in the function definition
-        documented_defined = [n for n in documented if n in parsed_params]
-        if defined_documented != documented_defined:
+        documented_defined = (n for n in documented if n in parsed_params)
+        if tuple(defined_documented) != tuple(documented_defined):
             yield cls.DOC106, ()
 
         # Check for missing parameter keys in docstring
@@ -408,7 +423,7 @@ class Violations:
                 # Missing sep or invalid context (section_key_ctx is True)
                 yield cls.DOC002, (section_key,)
             elif section_key in return_set and not description:
-                # :return:´ without description
+                # ´:return:´ without description
                 yield cls.DOC002, (section_key,)
             elif section_key == rtype_key and not doc_return_type:  # ´:rtype:´ without type
                 yield cls.DOC002, (section_key,)
@@ -697,7 +712,7 @@ def parse_docs(node, filename, /):
         after_lineno = docs_end_lineno + 1
         # Use linecache instead of ast to get the next line after the docstring correctly, because ast skips comments.
         after_line = linecache.getline(filename, after_lineno)
-        missing_blank_line_after = after_line.strip() != ""
+        missing_blank_line_after = bool(after_line.strip())  # True if the line is not blank
     else:
         missing_blank_line_after = False
 
@@ -894,14 +909,16 @@ def walk(paths, ignore_dirs, /):
             # return it even if it does not end with .py
             yield path
         elif path.is_dir():
-            for root, directories, files in path.walk():
+            for root, directories, files in os.walk(path):
                 # Remove ignored directories
                 for candidate in frozenset(directories).intersection(ignore_dirs):
                     directories.remove(candidate)
 
                 for name in files:
                     if name.endswith(suffix):
-                        yield root / name
+                        yield pathlib.Path(os.path.join(root, name))
+        else:
+            logging.warning("Unknown file: %s", path)
 
 
 def dump_statistics(violations, /):
@@ -926,12 +943,15 @@ def dump_file(violations, quiet, path, /):
 
 def dump_version():
     path = pathlib.Path(__file__).with_name("pyproject.toml")
-    if path.exists():  # standalone script
+    if tomllib and path.exists():  # standalone script
         with path.open("rb") as f:
             version = tomllib.load(f)["project"]["version"]
     else:  # installed package
-        version = importlib.metadata.version("sphinx-linter")
-    print("sphinx-linter {}".format(version))
+        try:
+            version = importlib.metadata.version(APP_NAME)
+        except importlib.metadata.PackageNotFoundError:
+            version = "unknown"
+    print("{} {}".format(APP_NAME, version))
 
 
 def get_config_start_dir(paths, /):
@@ -953,6 +973,12 @@ def load_config(config_file, check_files, /):
     """
 
     config = dict()
+
+    if not tomllib:
+        msg = "tomllib not available; config files cannot be loaded. Please use Python 3.11+ to enable this feature."
+        logging.warning(msg)
+        return config
+
     if config_file:
         paths = (pathlib.Path(config_file),)
     else:
@@ -962,13 +988,13 @@ def load_config(config_file, check_files, /):
 
     for path in paths:
         try:
-            data = tomllib.loads(path.read_text())
-        except tomllib.TOMLDecodeError:
-            continue  # pyproject is malformed, skip
+            data = tomllib.loads(path.read_text(encoding="utf-8"))
+        except (tomllib.TOMLDecodeError, PermissionError):
+            continue  # Invalid TOML or inaccessible file, skip it
         else:
-            if "tool" in data and "sphinx-linter" in data["tool"]:
+            if "tool" in data and APP_NAME in data["tool"]:
                 # Only use if it has [tool.sphinx-linter] section
-                config = data["tool"]["sphinx-linter"]
+                config = data["tool"][APP_NAME]
                 break
     return config
 
