@@ -14,7 +14,7 @@ def parse_content(data):
 
 @pytest.fixture(scope='module')
 def violations():
-    return sphinxlinter.Violations()
+    return sphinxlinter.Violations(enable=["ALL"])
 
 
 def chequer(content, violations, next_line=""):
@@ -237,7 +237,6 @@ def foo():
     assert result == expected
 
 
-
 def test_DOC004(violations):
     content = '''
 def foo(a):
@@ -336,6 +335,7 @@ def foo(a, b, c):
     """
     Title.
 
+    :param int a: description
     :raises ValueError: description
     :param int b: description
     :return: description
@@ -487,7 +487,7 @@ def foo(a):
 ])
 def test_DOC010_function_ignoring_descriptions_ws(section, violations):
     content = f'''
-def foo(a, b):
+def foo(a):
     """
     Title.
 
@@ -629,6 +629,20 @@ def foo(a):
     assert result == expected
 
 
+def test_DOC012_oneline_docstring(violations):
+    content = '''
+def foo(a):
+    """ Title.
+
+    :param str a: description
+    """
+'''
+
+    expected = ((3, 'DOC012', 'Leading whitespaces in first non-blank line ({!r})', (' Title.',)),)
+    result = tuple(chequer(content, violations))
+    assert result == expected
+
+
 def test_DOC012_ignore_when_not_docstring(violations):
     content = '''
 def foo(a):
@@ -640,27 +654,18 @@ def foo(a):
     assert result == expected
 
 
-def test_DOC012_ignore_if_blank_docstring(violations):
-    content = '''
+@pytest.mark.parametrize("docs", ["", " ", "  ", "  \n"])
+def test_DOC012_ignore_if_blank_docstring(docs, violations):
+    content = f'''
 def foo(a):
-    """  """
+    """{docs}"""
 '''
 
-    expected = ()
-    result = tuple(chequer(content, violations))
-    assert result == expected
-
-
-def test_DOC012_oneline_docstring(violations):
-    content = '''
-def foo(a):
-    """ Title.
-
-    :param str a: description
-    """
-'''
-
-    expected = ((3, 'DOC012', 'Leading whitespaces in first non-blank line ({!r})', (' Title.',)),)
+    if "\n" in docs:
+        # Raises DOC006 for trailing empty lines but not DOC012
+        expected = ((None, 'DOC006', 'Trailing empty lines', ()),)
+    else:
+        expected = ()
     result = tuple(chequer(content, violations))
     assert result == expected
 
@@ -755,9 +760,15 @@ def foo({params}):
 
     pass
 '''
+
+    # Raise DOC106 always, and DOC107 if any parameter is missing in the docstring
     expected = (
         (3, "DOC106", "Parameter order mismatch with signature", ()),
     )
+    if params == "a, b, c":
+        expected += (
+            (3, "DOC107", "Missing parameter in docstring ({!r})", ("c",)),
+        )
     result = tuple(chequer(content, violations))
     assert result == expected
 
@@ -772,6 +783,50 @@ def foo({params}):
 
     pass
 '''
+
+    # Raise DOC107 for each undocumented parameter, but not DOC106
+    if params == "a, b":
+        expected = (
+            (3, "DOC107", "Missing parameter in docstring ({!r})", ("a",)),
+        )
+    else:
+        expected = (
+            (3, "DOC107", "Missing parameter in docstring ({!r})", ("a",)),
+            (3, "DOC107", "Missing parameter in docstring ({!r})", ("c",)),
+        )
+
+    result = tuple(chequer(content, violations))
+    assert result == expected
+
+
+def test_DOC107(violations):
+    content = '''
+def foo(self, cls, a, b, c, *args, **kwargs):
+    """
+    :type a: str
+    """
+
+    pass
+'''
+    expected = (
+        (3, 'DOC107', 'Missing parameter in docstring ({!r})', ('b',)),
+        (3, 'DOC107', 'Missing parameter in docstring ({!r})', ('c',)),
+    )
+    result = tuple(chequer(content, violations))
+    assert result == expected
+
+
+def test_DOC107_ignore_if_are_none_documented(violations):
+    content = '''
+def foo(a, b, c):
+    """
+    Title.
+
+    :rtype: int
+    """
+
+    return 1
+'''
     expected = ()
     result = tuple(chequer(content, violations))
     assert result == expected
